/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.osgicdi.impl;

import com.sun.enterprise.deploy.shared.ArchiveFactory;
import org.glassfish.api.ActionReport;
import org.glassfish.internal.deployment.Deployment;
import org.glassfish.osgijavaeebase.AbstractOSGiDeployer;
import org.glassfish.osgijavaeebase.OSGiApplicationInfo;
import org.glassfish.osgijavaeebase.OSGiDeploymentRequest;
import org.glassfish.osgijavaeebase.OSGiUndeploymentRequest;
import org.glassfish.server.ServerEnvironmentImpl;
import org.jboss.weld.bootstrap.api.ServiceRegistry;
import org.jboss.weld.bootstrap.spi.BeanDeploymentArchive;
import org.jboss.weld.bootstrap.spi.Metadata;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import java.net.URL;
import java.util.Collection;
import java.util.Dictionary;
import java.util.logging.Logger;

import javax.enterprise.inject.spi.Extension;

/**
 * @author Tang Yong(tangyong@cn.fujitsu.com)
 */
public class OSGiCDIDeployer extends AbstractOSGiDeployer
 {
    private static Logger logger = Logger.getLogger(OSGiCDIDeployer.class.getPackage().getName());
    
    private BeanDeploymentArchive beanDeploymentArchive;

    public OSGiCDIDeployer(BundleContext ctx) {
        super(ctx, Integer.MIN_VALUE);
    }

    public OSGiUndeploymentRequest createOSGiUndeploymentRequest(Deployment deployer, ServerEnvironmentImpl env, ActionReport reporter, OSGiApplicationInfo osgiAppInfo) {
        return new OSGiCDIUndeploymentRequest(deployer, env, reporter, osgiAppInfo);
    }

    public OSGiDeploymentRequest createOSGiDeploymentRequest(Deployment deployer, ArchiveFactory archiveFactory, ServerEnvironmentImpl env, ActionReport reporter, Bundle b) {
        return new OSGiCDIDeploymentRequest(deployer, archiveFactory, env, reporter, b);
    }

    public boolean handles(Bundle bundle) {
        return isPlainOSGiCDIBundle(bundle);
    }

    /**
     * Determines if a bundle represents a plain vanilla OSGi/CDI bundle or not.
     * We determine this by the following rules
     * 1) whether having EXPORT_EJB in manifest header or not.
     * 2) whether being a WAB or not
     * 3) whether having a beans.xml in META-INF or not
     * Currently, in the method implementation, if 1)is false && 2) is false && 3)is true,
     * then, it is a plain OSGi/CDI bundle. However, the above rules maybe improved in the future 
     * @param b
     * @return
     */
    private boolean isPlainOSGiCDIBundle(Bundle b) {
        final Dictionary<?, ?> headers = b.getHeaders();
        URL url = b.getEntry(Constants.BEANS_XML_PATH);
        return headers.get(Constants.EXPORT_EJB) == null && 
        		headers.get(Constants.WEB_CONTEXT_PATH) == null && url != null &&
                  headers.get(org.osgi.framework.Constants.FRAGMENT_HOST) == null;
    }
}
